import misc
from sprites import Player
import pygame
from constants import *


class AItemplate:
    def __init__(self, spawnpoint, spec_points, platforms, all_sprites):
        self.spawnpoint = spawnpoint
        self.spec_points = spec_points
        self.platforms = platforms
        self.done = False
        self.all_sprites = all_sprites  # for rendering purposes only
        self.found_path = False
        self.path = []

    def prepare(self):
        pass

    def next_step(self):
        pass

    def update_after_iteration(self):
        pass

    def reset_positions(self):
        pass


class AIPlayer(Player):
    def __init__(self, spawnpoint):
        super(AIPlayer, self).__init__(spawnpoint)
        self.cur_color = START_COLOUR
        self.surf.fill(self.cur_color)
        self.spec_idx = 0

    def update(self, pressed_keys, platforms, spec_points):
        # expected pressed_keys is different structure than pressed_keys in class Player
        self.prev_pos = self.rect.copy()
        self.rect.move_ip(0, self.gravity)
        self.rect.move_ip(self.momentum["sideways"], self.momentum["up_down"])

        # Update momentum
        self.momentum["up_down"] += 1  # Fall faster
        if self.momentum["sideways"] > 0:
            if self.momentum["sideways"] < 2:
                self.momentum["sideways"] = 0

            self.momentum["sideways"] -= 2
        elif self.momentum["sideways"] < 0:
            if self.momentum["sideways"] > -2:
                self.momentum["sideways"] = 0
            self.momentum["sideways"] += 2

        # Check for user input and move accordingly
        if K_UP in pressed_keys and self.can_jump:
            self.momentum["up_down"] = self.max_jump_height
            self.rect.move_ip(0, -self.speed)
        # Check for jump input
        if K_UP in pressed_keys and self.can_jump:
            if self.jump_timer == 0:  # just started jumping
                self.jump_timer = pygame.time.get_ticks()
            else:  # already jumping
                elapsed_time = pygame.time.get_ticks() - self.jump_timer
                if elapsed_time > self.jump_duration:
                    elapsed_time = self.jump_duration
                jump_progress = elapsed_time / self.jump_duration
                jump_height = self.min_jump_height + jump_progress * (self.max_jump_height - self.min_jump_height)
                self.momentum["up_down"] = jump_height

        # Check for end of jump input
        if K_UP not in pressed_keys and self.jump_timer > 0:
            self.jump_timer = 0
            if self.momentum["up_down"] < -10:  # cap minimum jump height
                self.momentum["up_down"] = -10

        if K_DOWN in pressed_keys:
            self.rect.move_ip(0, self.speed)
        if K_LEFT in pressed_keys:
            self.rect.move_ip(-self.speed, 0)
            self.momentum["sideways"] -= self.speed // 2
        if K_RIGHT in pressed_keys:
            self.rect.move_ip(self.speed, 0)
            self.momentum["sideways"] += self.speed // 2

        # Limit momentum
        if self.momentum["sideways"] > self.top_speed:
            self.momentum["sideways"] = self.top_speed
        if self.momentum["sideways"] < -self.top_speed:
            self.momentum["sideways"] = -self.top_speed
        if self.momentum["up_down"] > self.top_speed:
            self.momentum["up_down"] = self.top_speed

        # Keep player on the screen
        if self.rect.left < 0:
            self.rect.left = 0
            self.stop_momentum()
        if self.rect.right > SCREEN_WIDTH:
            self.rect.right = SCREEN_WIDTH
            self.stop_momentum()
        if self.rect.top <= 0:
            self.rect.top = 0
            self.stop_momentum()
        if self.rect.bottom >= SCREEN_HEIGHT:
            self.rect.bottom = SCREEN_HEIGHT
            self.stop_momentum()

        self.platform_collision_fix(platforms)

        # touched finish in past
        if self.spec_idx == len(spec_points):
            self.cur_color = (255, 255, 255)
            self.surf.fill(self.cur_color)

        # touched any special point
        elif pygame.Rect.colliderect(self.rect, spec_points[self.spec_idx]):
            self.spec_idx += 1
            # change color after getting to spec_point
            self.cur_color = misc.change_color(self.cur_color)
            self.surf.fill(self.cur_color)

    def next_step(self, platforms, spec_points):
        pass

    def reset_self(self):
        self.cur_color = START_COLOUR
        self.surf.fill(self.cur_color)
        self.spec_idx = 0
