import pygame

import misc
import sprites as sp
from misc import prepare_save_map

from pygame.locals import (
    K_ESCAPE,
    KEYDOWN,
    QUIT,
    MOUSEBUTTONDOWN,
    MOUSEBUTTONUP,
    K_s,
    K_f,
    K_c,
    K_x,
)

# Define constants for the screen width and height
SCREEN_WIDTH = 800
SCREEN_HEIGHT = 600

# S - add start
# C - add checkpoint
# F - add finish
# X - save map
# ESCAPE - exit without saving
# create platforms by holding LMB (left mouse button) and dragging
# remove by clicking with RMB


def add_platform(pos1, pos2, groups):
    size = abs(pos1[0] - pos2[0]), abs(pos1[1] - pos2[1])
    position = min(pos1[0], pos2[0]), min(pos1[1], pos2[1])

    new_platform = sp.Platform(position, size)
    new_platform.add(groups)


def add_special(pos, type, groups):
    new_special = sp.SpecialPoints(type, pos)
    new_special.add(groups)


if __name__ == "__main__":
    # Initialize pygame
    pygame.init()

    # Set up the clock for a decent framerate
    clock = pygame.time.Clock()

    # Create the screen object
    # The size is determined by the constant SCREEN_WIDTH and SCREEN_HEIGHT
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))

    # Variable to keep the main loop running
    running = True

    all_sprites = pygame.sprite.Group()
    platforms = pygame.sprite.Group()
    special_points = pygame.sprite.Group()

    # 3 platforms
    new_platform = sp.Platform((185, 170), (30, 60))
    platforms.add(new_platform)
    all_sprites.add(new_platform)

    new_platform = sp.Platform((0, SCREEN_HEIGHT - 40), (SCREEN_WIDTH, 40))
    platforms.add(new_platform)
    all_sprites.add(new_platform)

    new_platform = sp.Platform((SCREEN_WIDTH // 2, SCREEN_HEIGHT // 4), (10, SCREEN_HEIGHT // 2))
    platforms.add(new_platform)
    all_sprites.add(new_platform)

    mouse_pos_start = (0, 0)
    mouse_pos_stop = (0, 0)

    making_new_platform = False
    show_platform = None
    # Main loop
    while running:
        # for loop through the event queue
        for event in pygame.event.get():
            # Check for KEYDOWN event
            if event.type == KEYDOWN:
                # If the Esc key is pressed, then exit the main loop
                if event.key == K_ESCAPE:
                    running = False
                if event.key == K_s:
                    add_special(pygame.mouse.get_pos(), "Start", [all_sprites, special_points])
                if event.key == K_c:
                    add_special(pygame.mouse.get_pos(), "Checkpoint", [all_sprites, special_points])
                if event.key == K_f:
                    add_special(pygame.mouse.get_pos(), "Finish", [all_sprites, special_points])
                if event.key == K_x:
                    playground = prepare_save_map(special_points, platforms)
                    level_name = input("Enter name of created level: ")
                    misc.save_map(playground, level_name)
                    print("level saved")

            # Check for QUIT event. If QUIT, then set running to false.
            elif event.type == QUIT:
                if input("want to save map? (y/n): ").lower() == 'y':
                    playground = prepare_save_map(special_points, platforms)
                    level_name = input("Enter name of created level: ")
                    misc.save_map(playground, level_name)
                    print("level saved")
                running = False

            if event.type == MOUSEBUTTONDOWN:
                if pygame.mouse.get_pressed() == (True, False, False):  # left mouse button
                    mouse_pos_start = event.pos
                    making_new_platform = True
                    show_platform = sp.Platform(mouse_pos_start, (40, 40), color=(200, 200, 0))

                    all_sprites.add(show_platform)

                else:
                    if show_platform:
                        show_platform.kill()
                        show_platform = None
                        making_new_platform = False

            if event.type == MOUSEBUTTONUP:
                mouse_pos_stop = pygame.mouse.get_pos()
                # print(mouse_pos_start, mouse_pos_stop)
                if pygame.mouse.get_pressed() == (False, False, False) and making_new_platform:
                    show_platform.kill()
                    add_platform(mouse_pos_start, mouse_pos_stop, [all_sprites, platforms])
                    making_new_platform = False

        if pygame.mouse.get_pressed() == (False, False, True):
            for s in all_sprites:
                if s.rect.collidepoint(pygame.mouse.get_pos()):
                    s.kill()

        if making_new_platform:
            # horrible way of doing it but oh well
            mouse_pos = pygame.mouse.get_pos()
            size = abs(mouse_pos_start[0] - mouse_pos[0]), abs(mouse_pos_start[1] - mouse_pos[1])
            position = min(mouse_pos_start[0], mouse_pos[0]), min(mouse_pos_start[1], mouse_pos[1])
            show_platform.resize(size, position)
            # show_platform.kill()
            # show_platform = sp.Platform(position, size, color=(200, 200, 0))
            # all_sprites.add(show_platform)

            # show_platform.rect.center = mouse_pos
            # show_platform.rect.size = size

            # show_platform.rect.inflate_ip(1, 1)

            # show_platform.rect.height = size[0]
            # show_platform.rect.width = size[1]

        # Get the set of keys pressed and check for user input
        pressed_keys = pygame.key.get_pressed()

        # Fill the screen with black
        screen.fill((0, 0, 0))

        # Draw all sprites
        for entity in all_sprites:
            screen.blit(entity.surf, entity.rect)

        # Update the display
        pygame.display.flip()

        # Ensure program maintains a rate of 30 frames per second
        clock.tick(30)
