import pygame

import misc
import sprites as sp
from constants import *

if __name__ == "__main__":
    # Initialize pygame
    print("Choose map")
    playground, level_name = misc.load_map()

    pygame.init()

    # Set up the clock for a decent framerate
    clock = pygame.time.Clock()

    # Create the screen object
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))

    player = sp.Player(spawnpoint=playground.start)

    # Variable to keep the main loop running
    running = True

    # Create groups to hold enemy sprites and all sprites
    # - platforms is used for collision detection and position updates
    # - poi == spec_idx-of-interest used for checkpoints and finish
    # - all_sprites is used for rendering
    platforms = pygame.sprite.Group()
    poi = pygame.sprite.Group()
    all_sprites = pygame.sprite.Group()
    all_sprites.add(player)
    poi_list = []
    next_poi_idx = 0

    for platform in playground.platforms:
        new_platform = sp.Platform(platform[0], platform[1])
        platforms.add(new_platform)
        all_sprites.add(new_platform)

    for checkpoint in playground.checkpoints:
        new_checkpoint = sp.PointOfInterest("Checkpoint", checkpoint)
        poi.add(new_checkpoint)
        all_sprites.add(new_checkpoint)
        poi_list.append(new_checkpoint)

    finish = sp.PointOfInterest("Finish", playground.finish)
    poi.add(finish)
    poi_list.append(finish)
    all_sprites.add(finish)

    # Main loop
    while running:
        # for loop through the event queue
        for event in pygame.event.get():
            # Check for KEYDOWN event
            if event.type == KEYDOWN:
                # If the Esc key is pressed, then exit the main loop
                if event.key == K_ESCAPE:
                    running = False
            # Check for QUIT event. If QUIT, then set running to false.
            elif event.type == QUIT:
                running = False

        # Get the set of keys pressed and check for user input
        pressed_keys = pygame.key.get_pressed()

        # Get the set of keys pressed and check for user input
        mouse_pressed = pygame.mouse.get_pressed()

        # Update the player sprite based on user keypresses
        player.update(pressed_keys, platforms, poi)

        # Fill the screen with black
        screen.fill((0, 0, 0))
        # Draw the player on the screen
        screen.blit(player.surf, player.rect)

        # Draw all sprites
        for entity in all_sprites:
            screen.blit(entity.surf, entity.rect)

        # Check if any enemies have collided with the player
        collided = pygame.sprite.spritecollideany(player, poi)
        if pygame.sprite.spritecollideany(player, poi):
            # If so, then remove the player and stop the loop
            if collided == poi_list[next_poi_idx]:
                collided.kill()
                next_poi_idx += 1
                if next_poi_idx == len(poi_list):
                    running = False

        # Update the display
        pygame.display.flip()

        # Ensure program maintains a rate of 30 frames per second
        clock.tick(30)
